# -*- coding: utf-8 -*-
"""
Code that goes along with the 'user guide' in the BVR docs.

@author: Rebecca Kohlhaas
"""

import numpy as np
import pandas as pd
import sys
import joblib
import matplotlib
#matplotlib.use('agg')

# Add BayesValidRox path
sys.path.append("../../src/")
from bayesvalidrox.surrogate_models.inputs import Input
from bayesvalidrox.surrogate_models.exp_designs import ExpDesigns
from bayesvalidrox.pylink.pylink import PyLinkForwardModel
from bayesvalidrox.surrogate_models.surrogate_models import MetaModel
from bayesvalidrox.surrogate_models.engine import Engine
from bayesvalidrox.post_processing.post_processing import PostProcessing

if __name__ == '__main__':
    #### Priors, input space and experimental design
    version = 1
    Inputs = Input()
    
    # Version 1
    if version == 1:
        Inputs.add_marginals()
        Inputs.Marginals[0].name = '$X$'    
        Inputs.Marginals[0].dist_type = 'unif'
        Inputs.Marginals[0].parameters =  [-5, 5]
        
    # Version 2
    if version == 2:
        inputParams = np.random.uniform(-5,5,100)
        Inputs.add_marginals()
        Inputs.Marginals[0].name = '$X$'
        Inputs.Marginals[0].input_data = inputParams

    # Create ExpDesign and generate samples
    ExpDesign = ExpDesigns(Inputs)
    ExpDesign.sampling_method = 'latin_hypercube'
    samples = ExpDesign.generate_samples(10)
    ExpDesign.plot_samples(samples)
    
    #### Models
    Model = PyLinkForwardModel()
    Model.link_type = 'Function'
    Model.py_file = 'model'
    Model.name = 'model'
    Model.Output.names = ['A', 'B']
    
    #output, samples = Model.run_model_parallel(samples, mp = True)
    
    #from model import model
    #out1 = model(samples)
    
    #### Training surrogate models
    MetaMod = MetaModel(Inputs)
    MetaMod.meta_model_type = 'aPCE'
    MetaMod.pce_reg_method = 'FastARD'
    MetaMod.pce_deg = 3
    MetaMod.pce_q_norm = 0.85
    
    ExpDesign.n_init_samples = 10
    ExpDesign.sampling_method = 'user'
    ExpDesign.X = samples
    
    Engine_ = Engine(MetaMod, Model, ExpDesign)
    Engine_.start_engine()
    Engine_.train_normal()
    
    mean, stdev = Engine_.eval_metamodel(nsamples = 10)
    mean, stdev = Engine_.MetaModel.eval_metamodel(samples)
    
    #### Active learning
    ExpDesign.n_new_samples = 1
    ExpDesign.n_max_samples = 14
    ExpDesign.mod_LOO_threshold = 1e-16
    
    ExpDesign.tradeoff_scheme = None
    
    ExpDesign.explore_method = 'random'
    ExpDesign.n_canddidate = 1000
    ExpDesign.n_cand_groups = 4
    
    ExpDesign.exploit_method = 'VarOptDesign'
    ExpDesign.util_func = 'EIGF'
    
    Engine_ = Engine(MetaMod, Model, ExpDesign)
    Engine_.start_engine()
    Engine_.train_sequential()
    
    #### Postprocessing
    PostProc = PostProcessing(Engine_)
    PostProc.valid_metamodel(n_samples=1)
    PostProc.check_accuracy(n_samples=10)
    PostProc.plot_moments()
    PostProc.sobol_indices()
    PostProc.plot_seq_design_diagnostics()